;(function(){

	_App.DataStore = function()
	{
		this.initialize();
	};

	_App.DataStore.prototype =
	{

		bulkRequestId 	: 1,
		metaStore     	: {},
		dataStore     	: {},
		itemStore     	: {},
		listenerStore 	: {},
		reverse       	: {'years': true},

		initialize : function()
		{
			this.listenerStore = {};
		},

		/**
		 * Set param for storage.
		 * @param param
		 * @param value
		 */
		setItem : function( param, value )
		{
			this.itemStore[param] = value;
		},

		/**
		 * Retrieve param from storage
		 * @param param
		 * @param defaultValue
		 * @returns
		 */
		getItem : function( param, defaultValue )
		{
			return this.itemStore[param]
				? this.itemStore[param]
				: defaultValue;
		},

		/**
		 * Remove param from storage
		 * @param param
		 */
		removeItem : function( param )
		{
			if (this.itemStore[param])
			{
				delete this.itemStore[param];
			}
		},

		/**
		 * Register an Event Listener.
		 * @param id
		 * @param dataId
		 * @param callback
		 * @param type
		 */
		listen : function( id, dataId, callback, type )
		{
			if (!this.listenerStore[dataId])
			{
				this.listenerStore[dataId] = {};
			}
			this.listenerStore[dataId][id] = {
					"callback": callback,
					"type": (type ? type : "UseOnce")
			};
			if (this.dataStore[dataId])
			{
				this.listenerStore[dataId][id]["callback"](dataId, this.dataStore[dataId]);
			}
		},

		/**
		 * Remove a registered Event Listener.
		 * @param id
		 * @param dataId
		 */
		removeListener : function( id, dataId )
		{
			if (this.listenerStore[dataId] && this.listenerStore[dataId][id])
			{
				delete this.listenerStore[dataId][id];
			}
		},

		/**
		 * Set dataset.
		 * @param dataId
		 * @param data
		 */
		setData : function( dataId, data, noSwap )
		{
			if (!noSwap && this.reverse[dataId])
			{
				data.reverse();
			}
			this.dataStore[dataId] = data;
			if (this.listenerStore[dataId])
			{
				for (var id in this.listenerStore[dataId])
				{
					this.listenerStore[dataId][id]["callback"](dataId, data);
					if ("UseOnce" == this.listenerStore[dataId][id]["type"])
					{
						delete this.listenerStore[dataId][id];
					}
				}
			}
		},

		/**
		 * Retrieve dataset.
		 * @param dataId
		 */
		getData : function ( dataId )
		{
			return undefined != this.dataStore[dataId]
				? this.dataStore[dataId]
				: [];
		},

		/**
		 * Extract data from api response and set to dataStore.
		 * @param dataId
		 * @param apiResponse
		 */
		setDataFromApiResponse : function( dataId, callback, apiResponse )
		{
			this.setData(dataId,
					(apiResponse.Data.DataSet)
						? apiResponse.Data.DataSet
						: apiResponse.Data
					);
			if (undefined != callback)
			{
				callback();
			}
		},

		/**
		 * Extract data from api response and set to dataStore.
		 * @param dataId
		 * @param apiResponse
		 */
		setDataGroupFromApiResponse : function( requests, callback, responses )
		{
			for (var i in requests)
			{
				this.setData(requests[i].dataId,
						(responses[i].Data.DataSet)
							? responses[i].Data.DataSet
							: responses[i].Data
				);
				if (undefined != requests[i].callback)
				{
					requests[i].callback();
				}
			}
			if (undefined != callback)
			{
				callback();
			}
		},

		/**
		 * Load a group of select lists.
		 * @param requests
		 * @param callback
		 */
		loadSelectListGroup : function( requests, callback )
		{
			var contracts = [];
			var req = [];
			for (var i in requests)
			{
				if (requests[i].isStatic && App.DataStore.dataStore[requests[i].dataId])
				{
					App.DataStore.setData(requests[i].dataId, App.DataStore.dataStore[requests[i].dataId], true);
					continue;
				}
				req.push(requests[i]);
				App.DataStore.metaStore[requests[i].dataId] = {};
				App.DataStore.metaStore[requests[i].dataId].data = requests[i].data
					? requests[i].data
					: {};
				App.DataStore.metaStore[requests[i].dataId].options = requests[i].options
					? requests[i].options
					: {};
				contracts.push({
					id: 'DataStore.' + requests[i].dataId,
					workspace: requests[i].workspace,
					task: requests[i].task,
					jobId: requests[i].jobId,
					data: requests[i].data,
					callback: requests[i].callback,
					errorCallback: requests[i].errorCallback
				});
			}
			if (0 < req.length)
			{
				App.API.getTasks(
						contracts,
						$.proxy(App.DataStore._loadDataGroup, this, req, callback)
				);
			}
		},

		_loadDataGroup : function( requests, callback )
		{
			var contracts = [];
			for (var i in requests)
			{
				contracts.push({
					id: 'DataStore.' + requests[i].dataId,
					data: App.DataStore.metaStore[requests[i].dataId].data,
					options: App.DataStore.metaStore[requests[i].dataId].options
				});
			}
			this.bulkRequestId++;
			App.API.execTasks(
					contracts,
					$.proxy(App.DataStore.setDataGroupFromApiResponse, this, requests, callback)
			);
		},

		/**
		 * Load a select list dataset via api.
		 * @param dataId
		 * @param isStatic
		 * @param workspace
		 * @param task
		 * @param jobId
		 * @param data
		 * @param options
		 */
		loadSelectListData : function( dataId, isStatic, workspace, task, jobId, data, options, callback )
		{
			if (isStatic && App.DataStore.dataStore[dataId])
			{
				App.DataStore.setData(dataId, App.DataStore.dataStore[dataId], true);
				return;
			}
			App.DataStore.metaStore[dataId] = {};
			App.DataStore.metaStore[dataId].data = data
				? data
				: {};
			App.DataStore.metaStore[dataId].options = options
				? options
				: {};

			var taskContract = App.API.taskContract('DataStore.' + dataId);
			if (null == taskContract
				|| undefined == taskContract.Hash
				|| null == taskContract.Hash)
			{
				App.API.getTask(
						'DataStore.' + dataId, workspace, task, jobId,
						{},
						$.proxy(App.DataStore._loadData, this, dataId, isStatic, callback),
						_w.taskContractError
						);
			}
			else
			{
				App.DataStore._loadData(dataId, isStatic, callback);
			}
		},

		_loadData : function( dataId, isStatic, callback )
		{
			App.API.execTask(
					'DataStore.' + dataId,
					App.DataStore.metaStore[dataId].data,
					App.DataStore.metaStore[dataId].options,
					$.proxy(App.DataStore.setDataFromApiResponse, this, dataId, callback),
					_w.taskExecError
					);
		}

	};

})();